@@ -40,7 +40,7 @@ ajax '/ajax/data/queue/typeahead/action' => require_role admin => sub {
     my @core_plugins = @{ setting('worker_plugins') || [] };
     my @user_plugins = @{ setting('extra_worker_plugins') || [] };
 
-    # load worker plugins for our action
+    # scan worker plugins for our action
     foreach my $plugin (@user_plugins, @core_plugins) {
       $plugin =~ s/^X::/+App::NetdiscoX::Worker::Plugin::/;
       $plugin = 'App::Netdisco::Worker::Plugin::'. $plugin
@@ -89,7 +89,8 @@ ajax '/ajax/data/devicename/typeahead' => require_login sub {
     return '[]' unless setting('navbar_autocomplete');
 
     my $q = param('query') || param('term');
-    my $set = schema(vars->{'tenant'})->resultset('Device')->search_fuzzy($q);
+    my $set = schema(vars->{'tenant'})->resultset('Device')
+      ->search_fuzzy($q)->search(undef, {rows => setting('max_typeahead_rows')});
 
     content_type 'application/json';
     to_json [map {encode_entities($_->dns || $_->name || $_->ip)} $set->all];
@@ -97,7 +98,8 @@ ajax '/ajax/data/devicename/typeahead' => require_login sub {
 
 ajax '/ajax/data/deviceip/typeahead' => require_login sub {
     my $q = param('query') || param('term');
-    my $set = schema(vars->{'tenant'})->resultset('Device')->search_fuzzy($q);
+    my $set = schema(vars->{'tenant'})->resultset('Device')
+      ->search_fuzzy($q)->search(undef, {rows => setting('max_typeahead_rows')});
 
     my @data = ();
     while (my $d = $set->next) {
@@ -113,6 +115,48 @@ ajax '/ajax/data/deviceip/typeahead' => require_login sub {
     to_json \@data;
 };
 
+ajax '/ajax/data/devices/typeahead' => require_login sub {
+    my $q = param('device_rule') or return '[]';
+    my $mode = param('aclhost') || 'ip';
+    if ($mode eq 'dynamic') {
+        $mode = (($q =~ m/^\d/ or $q =~ m/:/) ? 'ip' : 'name');
+    }
+
+    content_type 'application/json';
+    my @data = ();
+
+    # TODO add in entries from the database
+    my @host_groups = sort {$a cmp $b}
+                      grep {$_ !~ m/^synthesized_group_/}
+                      keys %{ setting('host_groups')};
+
+    # if q starts group: then search for host groups (excluding synthesized)
+    if ($q =~ m/^(?:group:|acl:)/i) {
+       return to_json [ map { 'group:'. $_ } @host_groups ];
+    }
+
+    if (scalar grep { $_ =~ m/\Q$q\E/i } @host_groups) {
+        push @data, map { 'group:'. $_ }
+                    grep { $_ =~ m/\Q$q\E/i } @host_groups
+    }
+
+    my $set = schema(vars->{'tenant'})->resultset('Device')
+      ->search_fuzzy($q)->search(undef, {rows => setting('max_typeahead_rows')});
+
+    while (my $d = $set->next) {
+        my $name = ($d->dns || $d->name);
+
+        if (not $name or $mode eq 'ip') {
+            push @data, { value => $d->ip, label => ($name ? sprintf('%s (%s)', $d->ip, $name) : $d->ip)  };
+        }
+        elsif ($mode eq 'name') {
+            push @data, { value => $name, label => sprintf('%s (%s)', $name, $d->ip) };
+        }
+    }
+
+    return to_json \@data;
+};
+
 ajax '/ajax/data/port/typeahead' => require_login sub {
     my $dev  = param('dev1')  || param('dev2');
     my $port = param('port1') || param('port2');
@@ -140,7 +184,7 @@ ajax '/ajax/data/subnet/typeahead' => require_login sub {
     $q = "$q\%" if $q !~ m/\%/;
     my $nets = schema(vars->{'tenant'})->resultset('Subnet')->search(
            { 'me.net::text'  => { '-ilike' => $q }},
-           { columns => ['net'], order_by => 'net' } );
+           { columns => ['net'], order_by => 'net', rows => setting('max_typeahead_rows') } );
 
     content_type 'application/json';
     to_json [map {$_->net} $nets->all];
