@@ -17,11 +17,6 @@ register_worker({ phase => 'check' }, sub {
   return Status->error("discover failed: no device param (need -d ?)")
     if $device->ip eq '0.0.0.0';
 
-  # runner has already called get_device to promote $job->device
-  return $job->cancel("fresh discover cancelled: $device already known")
-    if $device->in_storage
-       and ($job->subaction eq 'with-nodes' and not $job->username);
-
   return Status->info("discover skipped: $device is not discoverable")
     unless is_discoverable_now($device);
 
